//********************************************************************
//  Analyze.java       Author: Ryo (Rio) Akasaka
//  
//  Last modified: Mon Feb 25, 2007
//
//  This class allows the user to analyze a file and output data based
//  on alphabetical order, frequency, frequency based on initials, as 
//  well as alphabetical order of initials. It uses the sort algorithm 
//  provided by the Collections library within Java. It implements an
//  ArrayList, as well as an accompanying comparator that defines the
//  specific parameter to be compared. Though seemingly redundant, 
//  each of the comparators functions independently: the SortFreq class
//  sorts by the frequency found next to each name, the SortAlpha class
//  sorts alphabetically by the name in the input data file, the 
//  SortInitials class sorts by the initial available within the data,
//  and the SortTotal class sorts by the total sum of frequencies 
//  calculated for each initial.
//
//  This program, as an updated version of the previous program, uses
//  two ArrayLists, so as to better organize the two different parame-
//  ters by which the sorting is done: by Name and Initial.
//
//  Otherwise, most of this program is identical to the previous one.
//  See the readme for further details, as well as the comments on the
//  top of each class file.
//********************************************************************

import java.util.Scanner;
import java.io.*;
import java.util.*;

public class Analyze
{
  public static void main (String[] args) throws IOException
  {
    Scanner filein;
    String filename, name;
    String option = "m";
    int num;
    
    Scanner keyin = new Scanner(System.in);
    System.out.println("\n\tThis program will read a file of names and" +
                       "\n\tnumber of occurences and  display them  in" +
		       "\n\torder of rank, initial, or in  alphabetical" +
		       "\n\torder");
    
    System.out.println();
    System.out.print("Please enter the file name to process: ");
    filename = keyin.nextLine();
    filein = new Scanner (new File(filename));
    PrintWriter fileout = new PrintWriter(
					  new FileWriter(filename + ".out"));
    System.out.println();

  //-----------------------------------------------------------------
  //  Read file and save into comparable list
  //-----------------------------------------------------------------
    int count = 0;
    int numberItems = new Integer(filein.next()); // Read first line

    ArrayList<Line> list = new ArrayList<Line>(numberItems);
    ArrayList<Init> initials = new ArrayList<Init>(26);
    Object [] namelist = new Object [numberItems]; // Contains a list of objects that are sorted alphabetically
    Object [] freqlist = new Object [numberItems]; // Contains a list of objects that are sorted by frequency
    Object [] initialist = new Object [26]; // Contains a list of initials that are sorted by frequency
    Object [] initalph = new Object [26]; // Contains a list of initials that are sorted alphabetically

    while (filein.hasNext()) {
	name = new String(filein.next());
	num = new Integer(filein.next());
	Line value = new Line(name, num); 
       list.add(value);
	count++;
      }

  //-----------------------------------------------------------------
  //  Sort by frequency
  //-----------------------------------------------------------------  
    Comparator<Line> freq = new SortFreq();  
    Collections.sort(list, freq);
    for (int i =0; i <count; i++) {
      ((Line) list.get(i)).setRank(i+1);
      freqlist[i] = ((Line) list.get(i));
    }
    
  //-----------------------------------------------------------------
  //  Sort by name alphabetically
  //-----------------------------------------------------------------
    Comparator<Line> alph = new SortAlpha();  
    Collections.sort(list, alph);
    for (int i =0; i <count; i++) {
      namelist[i] = ((Line) list.get(i));
      char init = (((Line) namelist[i]).getInitials());
      int asciinum = (int) (init);
      initials.add(new Init());
      ((Init) initials.get(asciinum - 65)).addInitials(init);
      ((Init) initials.get(asciinum - 65)).addNum(((Line) namelist[i]).getNum());
    }
    
  //-----------------------------------------------------------------
  //  Sort by intials/frequency
  //-----------------------------------------------------------------
    Comparator<Init> totalfreq = new SortTotal();  
    Collections.sort(initials, totalfreq);
    for (int i =0; i <26; i++) {  
      ((Init) initials.get(i)).setRank(i+1);
      initialist[i] = ((Init) initials.get(i));
    }	

  //-----------------------------------------------------------------
  //  Sort by initials/alphabetical
  //-----------------------------------------------------------------
    Comparator<Init> initsort = new SortInitials();  
    Collections.sort(initials, initsort);
    for (int i =0; i <26; i++) { 
      initalph[i] = ((Init) initials.get(i)); 
    }	
    
    while (!option.equalsIgnoreCase("q")) {
      System.out.println("Please choose one: " +
			 "\n\ta -- for names in alphabetical order" +
			 "\n\tr -- for names in rank order" +
			 "\n\ti -- for initials in alphabetical order" + 
			 "\n\tj -- for intitials in rank order" + 
			 "\n\tm -- for a repeat of this menu" + 
			 "\n\tq -- to quit this program"); 
      
      System.out.print(">> ");
      option = keyin.next();
      System.out.println();
      
      Reporter printer = new Reporter();	

    if (option.equalsIgnoreCase("r")) { printer.display(option, freqlist); }
    if (option.equalsIgnoreCase("a")) { printer.display(option, namelist); }
    if (option.equalsIgnoreCase("j")) { printer.display(option, initialist); }
    if (option.equalsIgnoreCase("i")) { printer.display(option, initalph); }

    //-----------------------------------------------------------------
    //  Quit
    //-----------------------------------------------------------------
    if (option.equalsIgnoreCase("q")) {
      System.out.println("Have a nice day!");	
      System.exit(0);
    }
    System.out.println();
    }
  }
}